@@ -5,15 +5,16 @@
 // http://opensource.org/licenses/MIT>, at your option. This file may not be
 // copied, modified, or distributed except according to those terms.
 
-use deadlock;
-use lock_api::RawMutex as RawMutexTrait;
-use mutex::MutexGuard;
+use crate::mutex::MutexGuard;
+use crate::raw_mutex::{RawMutex, TOKEN_HANDOFF, TOKEN_NORMAL};
+use crate::{deadlock, util};
+use core::{
+    fmt, ptr,
+    sync::atomic::{AtomicPtr, Ordering},
+};
+use lock_api::RawMutex as RawMutex_;
 use parking_lot_core::{self, ParkResult, RequeueOp, UnparkResult, DEFAULT_PARK_TOKEN};
-use raw_mutex::{RawMutex, TOKEN_HANDOFF, TOKEN_NORMAL};
-use std::sync::atomic::{AtomicPtr, Ordering};
 use std::time::{Duration, Instant};
-use std::{fmt, ptr};
-use util;
 
 /// A type indicating whether a timed wait on a condition variable returned
 /// due to a time out or not.
@@ -23,7 +24,7 @@ pub struct WaitTimeoutResult(bool);
 impl WaitTimeoutResult {
     /// Returns whether the wait was known to have timed out.
     #[inline]
-    pub fn timed_out(&self) -> bool {
+    pub fn timed_out(self) -> bool {
         self.0
     }
 }
@@ -77,9 +78,13 @@ impl WaitTimeoutResult {
 /// // wait for the thread to start up
 /// let &(ref lock, ref cvar) = &*pair;
 /// let mut started = lock.lock();
-/// while !*started {
+/// if !*started {
 ///     cvar.wait(&mut started);
 /// }
+/// // Note that we used and if instead of a while loop above. This is only
+/// // possible because parking_lot's Condvar will never spuriously wake up.
+/// // This means that wait() will only return after notify_one or notify_all is
+/// // called.
 /// ```
 pub struct Condvar {
     state: AtomicPtr<RawMutex>,
@@ -88,7 +93,6 @@ pub struct Condvar {
 impl Condvar {
     /// Creates a new condition variable which is ready to be waited on and
     /// notified.
-    #[cfg(feature = "nightly")]
     #[inline]
     pub const fn new() -> Condvar {
         Condvar {
@@ -96,16 +100,6 @@ impl Condvar {
         }
     }
 
-    /// Creates a new condition variable which is ready to be waited on and
-    /// notified.
-    #[cfg(not(feature = "nightly"))]
-    #[inline]
-    pub fn new() -> Condvar {
-        Condvar {
-            state: AtomicPtr::new(ptr::null_mut()),
-        }
-    }
-
     /// Wakes up one blocked thread on this condvar.
     ///
     /// Returns whether a thread was woken up.
@@ -141,7 +135,6 @@ impl Condvar {
     }
 
     #[cold]
-    #[inline(never)]
     fn notify_one_slow(&self, mutex: *mut RawMutex) -> bool {
         unsafe {
             // Unpark one thread and requeue the rest onto the mutex
@@ -203,7 +196,6 @@ impl Condvar {
     }
 
     #[cold]
-    #[inline(never)]
     fn notify_all_slow(&self, mutex: *mut RawMutex) -> usize {
         unsafe {
             // Unpark one thread and requeue the rest onto the mutex
@@ -263,7 +255,7 @@ impl Condvar {
     /// This function will panic if another thread is waiting on the `Condvar`
     /// with a different `Mutex` object.
     #[inline]
-    pub fn wait<T: ?Sized>(&self, mutex_guard: &mut MutexGuard<T>) {
+    pub fn wait<T: ?Sized>(&self, mutex_guard: &mut MutexGuard<'_, T>) {
         self.wait_until_internal(unsafe { MutexGuard::mutex(mutex_guard).raw() }, None);
     }
 
@@ -293,7 +285,7 @@ impl Condvar {
     #[inline]
     pub fn wait_until<T: ?Sized>(
         &self,
-        mutex_guard: &mut MutexGuard<T>,
+        mutex_guard: &mut MutexGuard<'_, T>,
         timeout: Instant,
     ) -> WaitTimeoutResult {
         self.wait_until_internal(
@@ -397,7 +389,7 @@ impl Condvar {
     #[inline]
     pub fn wait_for<T: ?Sized>(
         &self,
-        mutex_guard: &mut MutexGuard<T>,
+        mutex_guard: &mut MutexGuard<'_, T>,
         timeout: Duration,
     ) -> WaitTimeoutResult {
         let deadline = util::to_deadline(timeout);
@@ -413,18 +405,18 @@ impl Default for Condvar {
 }
 
 impl fmt::Debug for Condvar {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.pad("Condvar { .. }")
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use crate::{Condvar, Mutex, MutexGuard};
     use std::sync::mpsc::channel;
     use std::sync::Arc;
     use std::thread;
     use std::time::{Duration, Instant};
-    use {Condvar, Mutex};
 
     #[test]
     fn smoke() {
@@ -626,7 +618,7 @@ mod tests {
         rx.recv().unwrap();
         let _g = m.lock();
         let _guard = PanicGuard(&*c);
-        let _ = c.wait(&mut m3.lock());
+        c.wait(&mut m3.lock());
     }
 
     #[test]
@@ -668,10 +660,393 @@ mod tests {
         let mut g = m.lock();
         while !c.notify_one() {
             // Wait for the thread to get into wait()
-            ::MutexGuard::bump(&mut g);
+            MutexGuard::bump(&mut g);
         }
         // The thread should have been requeued to the mutex, which we wake up now.
         drop(g);
         t.join().unwrap();
     }
+
+    #[test]
+    fn test_issue_129() {
+        let locks = Arc::new((Mutex::new(()), Condvar::new()));
+
+        let (tx, rx) = channel();
+        for _ in 0..4 {
+            let locks = locks.clone();
+            let tx = tx.clone();
+            thread::spawn(move || {
+                let mut guard = locks.0.lock();
+                locks.1.wait(&mut guard);
+                locks.1.wait_for(&mut guard, Duration::from_millis(1));
+                locks.1.notify_one();
+                tx.send(()).unwrap();
+            });
+        }
+
+        thread::sleep(Duration::from_millis(100));
+        locks.1.notify_one();
+
+        for _ in 0..4 {
+            assert_eq!(rx.recv_timeout(Duration::from_millis(500)), Ok(()));
+        }
+    }
+}
+
+/// This module contains an integration test that is heavily inspired from WebKit's own integration
+/// tests for it's own Condvar.
+#[cfg(test)]
+mod webkit_queue_test {
+    use crate::{Condvar, Mutex, MutexGuard};
+    use std::{collections::VecDeque, sync::Arc, thread, time::Duration};
+
+    #[derive(Clone, Copy)]
+    enum Timeout {
+        Bounded(Duration),
+        Forever,
+    }
+
+    #[derive(Clone, Copy)]
+    enum NotifyStyle {
+        One,
+        All,
+    }
+
+    struct Queue {
+        items: VecDeque<usize>,
+        should_continue: bool,
+    }
+
+    impl Queue {
+        fn new() -> Self {
+            Self {
+                items: VecDeque::new(),
+                should_continue: true,
+            }
+        }
+    }
+
+    fn wait<T: ?Sized>(
+        condition: &Condvar,
+        lock: &mut MutexGuard<'_, T>,
+        predicate: impl Fn(&mut MutexGuard<'_, T>) -> bool,
+        timeout: &Timeout,
+    ) {
+        while !predicate(lock) {
+            match timeout {
+                Timeout::Forever => condition.wait(lock),
+                Timeout::Bounded(bound) => {
+                    condition.wait_for(lock, *bound);
+                }
+            }
+        }
+    }
+
+    fn notify(style: NotifyStyle, condition: &Condvar, should_notify: bool) {
+        match style {
+            NotifyStyle::One => {
+                condition.notify_one();
+            }
+            NotifyStyle::All => {
+                if should_notify {
+                    condition.notify_all();
+                }
+            }
+        }
+    }
+
+    fn run_queue_test(
+        num_producers: usize,
+        num_consumers: usize,
+        max_queue_size: usize,
+        messages_per_producer: usize,
+        notify_style: NotifyStyle,
+        timeout: Timeout,
+        delay: Duration,
+    ) {
+        let input_queue = Arc::new(Mutex::new(Queue::new()));
+        let empty_condition = Arc::new(Condvar::new());
+        let full_condition = Arc::new(Condvar::new());
+
+        let output_vec = Arc::new(Mutex::new(vec![]));
+
+        let consumers = (0..num_consumers)
+            .map(|_| {
+                consumer_thread(
+                    input_queue.clone(),
+                    empty_condition.clone(),
+                    full_condition.clone(),
+                    timeout,
+                    notify_style,
+                    output_vec.clone(),
+                    max_queue_size,
+                )
+            })
+            .collect::<Vec<_>>();
+        let producers = (0..num_producers)
+            .map(|_| {
+                producer_thread(
+                    messages_per_producer,
+                    input_queue.clone(),
+                    empty_condition.clone(),
+                    full_condition.clone(),
+                    timeout,
+                    notify_style,
+                    max_queue_size,
+                )
+            })
+            .collect::<Vec<_>>();
+
+        thread::sleep(delay);
+
+        for producer in producers.into_iter() {
+            producer.join().expect("Producer thread panicked");
+        }
+
+        {
+            let mut input_queue = input_queue.lock();
+            input_queue.should_continue = false;
+        }
+        empty_condition.notify_all();
+
+        for consumer in consumers.into_iter() {
+            consumer.join().expect("Consumer thread panicked");
+        }
+
+        let mut output_vec = output_vec.lock();
+        assert_eq!(output_vec.len(), num_producers * messages_per_producer);
+        output_vec.sort();
+        for msg_idx in 0..messages_per_producer {
+            for producer_idx in 0..num_producers {
+                assert_eq!(msg_idx, output_vec[msg_idx * num_producers + producer_idx]);
+            }
+        }
+    }
+
+    fn consumer_thread(
+        input_queue: Arc<Mutex<Queue>>,
+        empty_condition: Arc<Condvar>,
+        full_condition: Arc<Condvar>,
+        timeout: Timeout,
+        notify_style: NotifyStyle,
+        output_queue: Arc<Mutex<Vec<usize>>>,
+        max_queue_size: usize,
+    ) -> thread::JoinHandle<()> {
+        thread::spawn(move || loop {
+            let (should_notify, result) = {
+                let mut queue = input_queue.lock();
+                wait(
+                    &*empty_condition,
+                    &mut queue,
+                    |state| -> bool { !state.items.is_empty() || !state.should_continue },
+                    &timeout,
+                );
+                if queue.items.is_empty() && !queue.should_continue {
+                    return;
+                }
+                let should_notify = queue.items.len() == max_queue_size;
+                let result = queue.items.pop_front();
+                std::mem::drop(queue);
+                (should_notify, result)
+            };
+            notify(notify_style, &*full_condition, should_notify);
+
+            if let Some(result) = result {
+                output_queue.lock().push(result);
+            }
+        })
+    }
+
+    fn producer_thread(
+        num_messages: usize,
+        queue: Arc<Mutex<Queue>>,
+        empty_condition: Arc<Condvar>,
+        full_condition: Arc<Condvar>,
+        timeout: Timeout,
+        notify_style: NotifyStyle,
+        max_queue_size: usize,
+    ) -> thread::JoinHandle<()> {
+        thread::spawn(move || {
+            for message in 0..num_messages {
+                let should_notify = {
+                    let mut queue = queue.lock();
+                    wait(
+                        &*full_condition,
+                        &mut queue,
+                        |state| state.items.len() < max_queue_size,
+                        &timeout,
+                    );
+                    let should_notify = queue.items.is_empty();
+                    queue.items.push_back(message);
+                    std::mem::drop(queue);
+                    should_notify
+                };
+                notify(notify_style, &*empty_condition, should_notify);
+            }
+        })
+    }
+
+    macro_rules! run_queue_tests {
+        ( $( $name:ident(
+            num_producers: $num_producers:expr,
+            num_consumers: $num_consumers:expr,
+            max_queue_size: $max_queue_size:expr,
+            messages_per_producer: $messages_per_producer:expr,
+            notification_style: $notification_style:expr,
+            timeout: $timeout:expr,
+            delay_seconds: $delay_seconds:expr);
+        )* ) => {
+            $(#[test]
+            fn $name() {
+                let delay = Duration::from_secs($delay_seconds);
+                run_queue_test(
+                    $num_producers,
+                    $num_consumers,
+                    $max_queue_size,
+                    $messages_per_producer,
+                    $notification_style,
+                    $timeout,
+                    delay,
+                    );
+            })*
+        };
+    }
+
+    run_queue_tests! {
+        sanity_check_queue(
+            num_producers: 1,
+            num_consumers: 1,
+            max_queue_size: 1,
+            messages_per_producer: 100_000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Bounded(Duration::from_secs(1)),
+            delay_seconds: 0
+        );
+        sanity_check_queue_timeout(
+            num_producers: 1,
+            num_consumers: 1,
+            max_queue_size: 1,
+            messages_per_producer: 100_000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        new_test_without_timeout_5(
+            num_producers: 1,
+            num_consumers: 5,
+            max_queue_size: 1,
+            messages_per_producer: 100_000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        one_producer_one_consumer_one_slot(
+            num_producers: 1,
+            num_consumers: 1,
+            max_queue_size: 1,
+            messages_per_producer: 100_000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        one_producer_one_consumer_one_slot_timeout(
+            num_producers: 1,
+            num_consumers: 1,
+            max_queue_size: 1,
+            messages_per_producer: 100_000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 1
+        );
+        one_producer_one_consumer_hundred_slots(
+            num_producers: 1,
+            num_consumers: 1,
+            max_queue_size: 100,
+            messages_per_producer: 1_000_000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        ten_producers_one_consumer_one_slot(
+            num_producers: 10,
+            num_consumers: 1,
+            max_queue_size: 1,
+            messages_per_producer: 10000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        ten_producers_one_consumer_hundred_slots_notify_all(
+            num_producers: 10,
+            num_consumers: 1,
+            max_queue_size: 100,
+            messages_per_producer: 10000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        ten_producers_one_consumer_hundred_slots_notify_one(
+            num_producers: 10,
+            num_consumers: 1,
+            max_queue_size: 100,
+            messages_per_producer: 10000,
+            notification_style: NotifyStyle::One,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        one_producer_ten_consumers_one_slot(
+            num_producers: 1,
+            num_consumers: 10,
+            max_queue_size: 1,
+            messages_per_producer: 10000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        one_producer_ten_consumers_hundred_slots_notify_all(
+            num_producers: 1,
+            num_consumers: 10,
+            max_queue_size: 100,
+            messages_per_producer: 100_000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        one_producer_ten_consumers_hundred_slots_notify_one(
+            num_producers: 1,
+            num_consumers: 10,
+            max_queue_size: 100,
+            messages_per_producer: 100_000,
+            notification_style: NotifyStyle::One,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        ten_producers_ten_consumers_one_slot(
+            num_producers: 10,
+            num_consumers: 10,
+            max_queue_size: 1,
+            messages_per_producer: 50000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        ten_producers_ten_consumers_hundred_slots_notify_all(
+            num_producers: 10,
+            num_consumers: 10,
+            max_queue_size: 100,
+            messages_per_producer: 50000,
+            notification_style: NotifyStyle::All,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+        ten_producers_ten_consumers_hundred_slots_notify_one(
+            num_producers: 10,
+            num_consumers: 10,
+            max_queue_size: 100,
+            messages_per_producer: 50000,
+            notification_style: NotifyStyle::One,
+            timeout: Timeout::Forever,
+            delay_seconds: 0
+        );
+    }
 }
